unit SendToClp;

interface

uses
  Windows, ActiveX, ComObj, ShlObj, ShellApi;

type
  TSendToClp = class(TComObject, IUnknown, IContextMenu, IShellExtInit)
  private
    fFileName: string;
    MyStr:string;
  protected
    {Declare IContextMenu methods here}
    function QueryContextMenu(Menu: HMENU; indexMenu, idCmdFirst, idCmdLast,
      uFlags: UINT): HResult; stdcall;
    function InvokeCommand(var lpici: TCMInvokeCommandInfo): HResult; stdcall;
    function GetCommandString(idCmd, uType: UINT; pwReserved: PUINT;
      pszName: LPSTR; cchMax: UINT): HResult; stdcall;
    {Declare IShellExtInit methods here}
    function IShellExtInit.Initialize = InitShellExt;
    function InitShellExt (pidlFolder: PItemIDList; lpdobj: IDataObject;
      hKeyProgID: HKEY): HResult; stdcall;
  end;

  TSendToClpFactory = class (TComObjectFactory)
  public
    procedure UpdateRegistry (Register: Boolean); override;
  end;

const
  Class_SendToClpMenu: TGUID ='{CDF05220-DB84-11D1-B9F1-004845400FAA}';

implementation

uses
  ComServ, Messages, SysUtils, Registry,Clipbrd;

// IShellExtInit method
function TSendToClp.InitShellExt(pidlFolder: PItemIDList;
  lpdobj: IDataObject; hKeyProgID: HKEY): HResult; stdcall;
var
  medium: TStgMedium;
  fe: TFormatEtc;
  i:longint;
  tmp:longint;
begin
  Result := E_FAIL;
  // check if the lpdobj pointer is nil
  setlength(mystr,0);
  if Assigned (lpdobj) then
  begin
    with fe do
    begin
      cfFormat := CF_HDROP;
      ptd := nil;
      dwAspect := DVASPECT_CONTENT;
      lindex := -1;
      tymed := TYMED_HGLOBAL;
    end;
    // transform the lpdobj data to a storage medium structure
    Result := lpdobj.GetData(fe, medium);
    if not Failed (Result) then
    begin
      // check if only one file is selected
      tmp:=  (DragQueryFile (medium.hGlobal, $FFFFFFFF, nil, 0) -1);
      for i:= 0 to tmp do
      begin
        ffilename:='';
        SetLength (fFileName, 1000);
        DragQueryFile (medium.hGlobal, i, PChar (fFileName), 1000);
        // realign string
        ffilename:=pchar(Ffilename);
        mystr:=concat(MyStr,TRIM(fFileName),#13#10);

      end;
    end;
    Result := NOERROR;
    ReleaseStgMedium(medium);
  end;
end;

// context menu methods

function TSendToClp.QueryContextMenu(Menu: HMENU;
  indexMenu, idCmdFirst, idCmdLast, uFlags: UINT): HResult;
begin
    InsertMenu (Menu, indexMenu,
      MF_STRING or MF_BYPOSITION, idCmdFirst,
      'Send to Clipboard');
    // Return number of menu items added
    Result := 1;

end;

function TSendToClp.InvokeCommand(var lpici: TCMInvokeCommandInfo): HResult;
Begin
  Result := NOERROR;
  // Make sure we are not being called by an application
  if HiWord(Integer(lpici.lpVerb)) <> 0 then
  begin
    Result := E_FAIL;
    Exit;
  end;
  // Make sure we aren't being passed an invalid argument number
  if LoWord(lpici.lpVerb) > 0 then
  begin
    Result := E_INVALIDARG;
    Exit;
  end;
  // execute the command specified by lpici.lpVerb.
  if LoWord(lpici.lpVerb) = 0 then
  begin
//      Clipboard.Clear;
      Clipboard.AsText:=MyStr;
      MyStr:='';
  end;
end;

function TSendToClp.GetCommandString(idCmd, uType: UINT; pwReserved: PUINT;
  pszName: LPSTR; cchMax: UINT): HRESULT;
begin

  if idCmd = 0 then
  begin
    // return help string for menu item
    strCopy (pszName, 'Add Filename to clipboard');
    Result := NOERROR;
  end
  else
    Result := E_INVALIDARG;
end;

{ TSendToClpFactory methods }

procedure TSendToClpFactory.UpdateRegistry(Register: Boolean);
var
  Reg: TRegistry;
begin
  inherited UpdateRegistry (Register);

  Reg := TRegistry.Create;
  try
    // register or remove the menu handler
    if Register then
      Reg.CreateKey (
        '\HKEY_CLASSES_ROOT\*\ShellEx\ContextMenuHandler\' +
        GUIDToString (Class_SendToClpMenu))
    else
      Reg.DeleteKey (
        '\HKEY_CLASSES_ROOT\*\ShellEx\ContextMenuHandler\' +
        GUIDToString (Class_SendToClpMenu));
  finally
    Reg.Free;
  end;
end;

initialization
  TSendToClpFactory.Create (
    ComServer, TSendToClp, Class_SendToClpMenu,
    'SendToClip', 'SendToClip Shell Extension',
    ciMultiInstance, tmApartment);

end.
